#include <functional>
#include <future>
#include <memory>
#include <string>
#include <sstream>

#include <rclcpp/rclcpp.hpp>

#include <geometry_msgs/msg/pose_stamped.hpp>
#include "geometry_msgs/msg/transform_stamped.h"

#include "tf2_ros/transform_listener.h"
#include <tf2_ros/buffer.h>

static const rclcpp::Logger LOGGER = rclcpp::get_logger("franka_pose_recorder");
using namespace std::chrono_literals;

class PoseRecoderNode : public rclcpp::Node
{
    public:
        using PoseStampedMsg = geometry_msgs::msg::PoseStamped;

        PoseRecoderNode()
        : Node("pose_recoder"), tf_buffer_(this->get_clock()),tf_listener_(tf_buffer_)
        {
            // Publisher for pose
            pose_publisher_ = this->create_publisher<geometry_msgs::msg::PoseStamped>("pose_recoder/current_pose", 10);


        // TF2
        tf2_ros::Buffer tf_buffer_{this->get_clock()};
        tf2_ros::TransformListener tf_listener_{tf_buffer_};
        tf2_timer_ = this->create_wall_timer(std::chrono::milliseconds(200), std::bind(&PoseRecoderNode::compute_endeffector_pose, this));

        }
    
    private:
        rclcpp::Publisher<geometry_msgs::msg::PoseStamped>::SharedPtr pose_publisher_;
        tf2_ros::Buffer tf_buffer_;
        tf2_ros::TransformListener tf_listener_;
        rclcpp::TimerBase::SharedPtr tf2_timer_;

        void compute_endeffector_pose()
        {
            try
            {
                geometry_msgs::msg::TransformStamped transform_stamped;
                transform_stamped = tf_buffer_.lookupTransform("panda_link1", "panda_link7", rclcpp::Time(0));
                
                geometry_msgs::msg::PoseStamped current_pose;
                current_pose.header.frame_id = "panda_link8";
                current_pose.header.stamp = transform_stamped.header.stamp;
                //Position
                current_pose.pose.position.x = transform_stamped.transform.translation.x;
                current_pose.pose.position.y = transform_stamped.transform.translation.y;
                current_pose.pose.position.z = transform_stamped.transform.translation.z;
                // Orientation
                current_pose.pose.orientation.x = transform_stamped.transform.rotation.x;
                current_pose.pose.orientation.y = transform_stamped.transform.rotation.y;
                current_pose.pose.orientation.z = transform_stamped.transform.rotation.z;
                current_pose.pose.orientation.w = transform_stamped.transform.rotation.w;
                pose_publisher_->publish(current_pose);
            }
            catch (tf2::TransformException &ex)
            {
                RCLCPP_WARN(this->get_logger(), "%s", ex.what());
            }
        }        
};

int main(int argc, char** argv)
{
    rclcpp::init(argc, argv);
    rclcpp::spin(std::make_shared<PoseRecoderNode>());
    rclcpp::shutdown();
    return 0;
}
